//
//  MAAddMediaSheet.mm
//  Medianno
//
//  Created by Matthias Neeracher on 8/11/11.
//  Copyright 2011 Matthias Neeracher. All rights reserved.
//

#import "MAAddMediaSheet.h"
#import "MAAppController.h"
#import "MADocument.h"

@implementation MAAddMediaSheet

static NSDictionary *   sDateHypotheses;

@synthesize media, dateFormats, selectedDateFormat;

- (id)init
{
 	self		= [super initWithWindowNibName:@"MAAddMedia"];

    if (!sDateHypotheses)
        sDateHypotheses = [[NSDictionary alloc] initWithContentsOfURL:
                           [[NSBundle mainBundle] URLForResource:@"DateHypotheses" withExtension:@"plist"]];
    
    return self;
}

- (NSString *)normalizedFileName:(NSURL *)url
{
    NSString * fileName = [[url lastPathComponent] stringByDeletingPathExtension];
    for (NSArray * norm in [sDateHypotheses objectForKey:@"normalize"]) {
        NSRegularExpression *   regexp  = [NSRegularExpression regularExpressionWithPattern:[norm objectAtIndex:0] options:0 error:nil];
        NSString *              templ   = [norm objectAtIndex:1];
        fileName                        = [regexp stringByReplacingMatchesInString:fileName options:0 
                                            range:NSMakeRange(0, [fileName length]) withTemplate:templ];
    }
    return fileName;
}

#define LOG_DATEGUESS 0
#if LOG_DATEGUESS
#define LOG_DG  NSLog
#else
#define LOG_DG(...) do {} while(0)
#endif

- (void)guessDateFormats:(NSArray *)urls
{
    [self window];
    [self willChangeValueForKey:@"dateFormats"];
    [dateFormats release];
    dateFormats  = nil;
    while ([dateFormatMenu numberOfItems] > 2)
        [dateFormatMenu removeItemAtIndex:2];
    
    if (![urls count])
        return;
    
    NSMutableArray * formats = [NSMutableArray array];
    
    NSString * fileName = [self normalizedFileName:[urls objectAtIndex:0]];
    LOG_DG(@"Representative File: %@\n", fileName);
    NSRange    nameRange= NSMakeRange(0, [fileName length]);
    //
    // Since date and time formats are largely orthogonal, and since for each regular expression, 
    // there are a number of possible date format hypotheses, we end up with 4 nested loops:
    //  - Regular expressions matching date portion
    //  - Regular expressions matching time portion
    //  - Date format matching date portion
    //  - Date format matching time portion
    //
    for (NSDictionary * dateHypo in [sDateHypotheses objectForKey:@"date"]) {
        NSString *              filterString    = [dateHypo objectForKey:@"regexp"];
        NSRegularExpression *   filter          = 
            [NSRegularExpression regularExpressionWithPattern:filterString options:0 error:nil];
        LOG_DG(@"  Filtering %@\n", filterString);
        if (![filter numberOfMatchesInString:fileName options:0 range:nameRange])
            continue; // Not going to match this one, regardless of time format
        for (NSDictionary * timeHypo in [sDateHypotheses objectForKey:@"time"]) {
            NSString * regexpString             = 
                [[dateHypo objectForKey:@"regexp"] stringByAppendingString:[timeHypo objectForKey:@"regexp"]];
            LOG_DG(@"    Trying %@\n", regexpString);
            NSRegularExpression * regexp        = 
                [NSRegularExpression regularExpressionWithPattern:regexpString options:0 error:nil];
            NSTextCheckingResult* match         = [regexp firstMatchInString:fileName options:0 range:nameRange];
            if (match && [match range].location != NSNotFound) {
                for (NSString * dateFmt in [dateHypo objectForKey:@"formats"]) {
                    for (NSString * timeFmt in [timeHypo objectForKey:@"formats"]) {
                        NSString * format       = [dateFmt stringByAppendingString:timeFmt];
                        LOG_DG(@"      Format %@\n", format);
                        NSString * dateFormat   = [regexp replacementStringForResult:match inString:fileName offset:0 template:format];
                        NSString * dateTitle    = [NSString stringWithFormat:NSLocalizedString(@"File Name Date", @"File Name yyyy-mm-dd"), dateFormat];
                        NSMenuItem * item       = [dateFormatMenu addItemWithTitle:dateTitle action:@selector(useFileNameDates:) keyEquivalent:@""];
                        [item setTag:[formats count]];
                        [formats addObject:[NSDictionary dictionaryWithObjectsAndKeys:
                                            dateFormat, @"format", 
                                            regexp,     @"regexp",
                                            nil]];
                    }
                }
            }
        }
    }
    if ([formats count] > 0)
        dateFormats = [formats retain];
    [self didChangeValueForKey:@"dateFormats"];
}

- (void)runWithParentWindow:(NSWindowController *)parent media:(NSArray *)mediaUrls
{
    [self guessDateFormats:mediaUrls];
    NSMutableArray * m = [[NSMutableArray alloc] initWithCapacity:[mediaUrls count]];
    for (NSURL * url in mediaUrls) 
        [m addObject:[NSMutableDictionary dictionaryWithObject:url forKey:@"url"]];
    media = m;
    [self useModificationDates:self];
    [NSApp beginSheet:[self window] modalForWindow:[parent window] didEndBlock:^(NSInteger returnCode) {
        if (returnCode == NSAlertFirstButtonReturn) {
            MADocument * doc = [parent document];
            [media enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
                NSURL *     url = [obj objectForKey:@"url"];
                NSString *  name= [[url lastPathComponent] stringByDeletingPathExtension];
                NSDate *    date= [obj objectForKey:@"date"];
                [doc addMediaURL:url name:name date:date];
            }];
        }
        [[self window] orderOut:self];
        [self autorelease];
        [media release];
        media = nil;
    }];
}

- (IBAction)useModificationDates:(id)sender
{
    [self setSelectedDateFormat:nil];
    [self willChangeValueForKey:@"media"];
    for (NSMutableDictionary * m in media) {
        NSDate *    modificationDate;
        if (![[m objectForKey:@"url"] getResourceValue:&modificationDate forKey:NSURLContentModificationDateKey error:nil])
            modificationDate = nil;
        [m setObject:(modificationDate ? modificationDate : [NSDate date]) forKey:@"date"];
    }
    [self didChangeValueForKey:@"media"];
}

- (IBAction)useFileNameDates:(id)sender
{
    [self setSelectedDateFormat:[dateFormats objectAtIndex:[sender tag]]];
    NSRegularExpression * regexp    = [selectedDateFormat objectForKey:@"regexp"];
    NSString            * format    = [selectedDateFormat objectForKey:@"format"];
    NSDateFormatter *     formatter = [[NSDateFormatter alloc] init];
    [formatter setDateFormat:format];
    
    [self willChangeValueForKey:@"media"];
    for (NSMutableDictionary * m in media) {
        NSString *              name = [self normalizedFileName:[m objectForKey:@"url"]];
        LOG_DG(@"File %@\n", name);
        NSTextCheckingResult *  match= [regexp firstMatchInString:name options:0 range:NSMakeRange(0, [name length])];
        NSDate *    modificationDate = nil;
        if (match && [match range].location != NSNotFound) {
            //
            // First and last capture is for leading and trailing context
            //
            NSRange dateRange       = [match rangeAtIndex:1];
            dateRange.location     += dateRange.length;
            dateRange.length        = [match rangeAtIndex:[match numberOfRanges]-1].location - dateRange.location;
            NSString * dateString   = [name substringWithRange:dateRange];
            modificationDate        = [formatter dateFromString:dateString];
            LOG_DG(@"  Date %@ %@\n", format, (modificationDate ? @"succeeded" : @"failed"));
        }
        if (!modificationDate)
            if (![[m objectForKey:@"url"] getResourceValue:&modificationDate forKey:NSURLContentModificationDateKey error:nil])
                modificationDate = nil;
        [m setObject:(modificationDate ? modificationDate : [NSDate date]) forKey:@"date"];
    }
    [self didChangeValueForKey:@"media"];
}

- (IBAction)endSheet:(id)sender
{
	[NSApp endSheet:[self window] returnCode:[sender tag]];
}

@end
