//
//  MADocument.m
//  Medianno
//
//  Created by Matthias Neeracher on 8/10/11.
//  Copyright 2011 Matthias Neeracher. All rights reserved.
//

#import "MADocument.h"
#import "MADocWindow.h"
#import "MAMedia.h"
#import "MAAnno.h"
#import "MATagDescription.h"

#pragma mark MADocument

@implementation MADocument

- (id)initWithType:(NSString *)typeName error:(NSError **)outError
{
    self = [super initWithType:typeName error:outError];
    return self;
}

- (NSString *)windowNibName
{
    return @"MADocument";
}

- (void)makeWindowControllers
{
    MADocWindow * windowController = [[MADocWindow alloc] initWithWindowNibName:[self windowNibName]];
    [self addWindowController:windowController];
    [windowController release];
}

+ (BOOL)autosavesInPlace
{
    return YES;
}

#pragma mark Media management

- (MAMedia *)addMediaURL:(NSURL *)url name:(NSString *)name date:(NSDate *)date
{
    NSString *      path        = [url absoluteString];
    NSNumber *      size;
    int64_t         fileSize;
    if ([url getResourceValue:&size forKey:NSURLFileSizeKey error:nil])
        fileSize    = [size longLongValue];
    else
        fileSize    = 0;
    NSTimeInterval  fileDate    = [date timeIntervalSinceReferenceDate];

    NSManagedObjectContext *moc     = [self managedObjectContext];
    MAMedia *               media   = nil;
    //
    // Look for existing file with same name, file size, and (preferably) date first.
    // Since the date may be parsed from the file name (and subject to a later fix),
    // we tolerate mismatches.
    //
    NSFetchRequest *        fetch   = [NSFetchRequest fetchRequestWithEntityName:@"MAMedia"];
    [fetch setPredicate:[NSPredicate predicateWithFormat:@"name == %@", name]];
    for (MAMedia * m in [moc executeFetchRequest:fetch error:nil]) 
        if (m.size == fileSize) {
            media = m;
            if (m.date == fileDate)
                break;
        }
    if (!media) {
        media = [NSEntityDescription insertNewObjectForEntityForName:@"MAMedia" inManagedObjectContext:moc];
        media.size  = fileSize;
        media.name  = name;
        media.notes = @"";
    }
    media.date  = fileDate;
    media.media = path;
    
    return media;
}

#pragma mark Annotation management

- (MAAnno *)addAnnotationForMedia:(MAMedia *)media location:(QTTime)location
{
    NSManagedObjectContext *moc = [self managedObjectContext];
    MAAnno * annotation = [NSEntityDescription insertNewObjectForEntityForName:@"MAAnno" inManagedObjectContext:moc];
    annotation.media    = media;
    annotation.location = QTStringFromTime(location);
    annotation.notes    = @"";
    
    return annotation;
}

#pragma mark Tag management

- (MATagDescription *)tagDescriptionForName:(NSString *)name
{
    NSManagedObjectContext *moc     = [self managedObjectContext];
    NSFetchRequest *        fetch   = [NSFetchRequest fetchRequestWithEntityName:@"MATagDescription"];
    [fetch setPredicate:[NSPredicate predicateWithFormat:@"name == %@", name]];
    NSArray * found = [moc executeFetchRequest:fetch error:nil];
    if (found && [found count]) {
        return [found objectAtIndex:0];
    } else {
        MATagDescription * tagDesc = [NSEntityDescription insertNewObjectForEntityForName:@"MATagDescription" inManagedObjectContext:moc];
        tagDesc.name = name;
        tagDesc.notes= @"";
    
        return tagDesc;
    }
}

- (MATagDescription *)tagDescriptionForName:(NSString *)name notes:(NSString *)notes
{
    MATagDescription * tag = [self tagDescriptionForName:name];
    
    if (notes && [notes length] && ![tag.notes length])
        [tag setNotes:notes];
    
    return tag;
}

- (NSArray *)tagNamesMatchingPrefix:(NSString *)prefix
{
    NSManagedObjectContext *moc     = [self managedObjectContext];
    NSFetchRequest *        fetch   = [NSFetchRequest fetchRequestWithEntityName:@"MATagDescription"];
    [fetch setPredicate:[NSPredicate predicateWithFormat:@"name BEGINSWITH[cd] %@", prefix]];

    NSArray * descs = [moc executeFetchRequest:fetch error:nil];
    NSMutableArray * names = [NSMutableArray arrayWithCapacity:[descs count]];
    for (MATagDescription * desc in descs)
        [names addObject:[desc name]];
    return names;
}

- (NSArray *)tagDescriptionsMatching:(NSString *)substring
{
    NSManagedObjectContext *moc     = [self managedObjectContext];
    NSFetchRequest *        fetch   = [NSFetchRequest fetchRequestWithEntityName:@"MATagDescription"];
    [fetch setPredicate:[NSPredicate predicateWithFormat:@"name CONTAINS[cd] %@ OR notes CONTAINS[cd] %@", 
                         substring, substring]];
    
    return [moc executeFetchRequest:fetch error:nil];    
}

- (NSSet *)allTags
{
    NSManagedObjectContext *moc     = [self managedObjectContext];
    NSFetchRequest *        fetch   = [NSFetchRequest fetchRequestWithEntityName:@"MATagDescription"];
    
    return [NSSet setWithArray:[moc executeFetchRequest:fetch error:nil]];
}

@end
